#!/usr/bin/Rscript
#PEcAn
#data
#pecan.nc, pecan.zip
#met.ED2, met.SIPNET, met.BIOCRO, met.DALEC, met.CLM45, met.PRELES, met.MAESPA, met.JULES, met.LINKAGES, met.FATES, met.GDAY, met.LPJGUESS, met.MAAT, clim, ed.zip, linkages, dalec

#send all output to stdout (incl stderr)
sink(stdout(), type="message")

# get command line arguments
args <- commandArgs(trailingOnly = TRUE)
if (length(args) < 2) {
  allargs <- commandArgs(trailingOnly = FALSE)
  myCommand <- sub('--file=', '', allargs[grep('--file=', allargs)])
  print(paste0("Usage:    ", myCommand, " xml_Input_File  Output_File [tempDirectory] [cacheDirectory]"))
  print(paste0("Example1: ", myCommand, " US-Dk3.pecan.nc US-Dk3.clim [/tmp/watever] [/tmp/cache]"))
  print(paste0("Example2: ", myCommand, " US-Dk3.pecan.zip US-Dk3.clim [/tmp/watever] [/tmp/cache]"))
  q()
}

# load required libraries
library(RPostgreSQL)
library(PEcAn.data.atmosphere)
library(PEcAn.DB)

# 1st argument is the input file
inputfile <- args[1]

# 2nd argument is the output file
outputfile <- args[2]

# 3rd argument is the temp folder
ifelse(length(args) > 2, tempDir <- args[3], tempDir <- ".")

# 4th argument is the cachefolder
ifelse(length(args) > 3, cacheDir <- args[4], cacheDir <- tempDir)
cacheDir <- "/home/polyglot/cache/PEcAn"
tmpfolder <- "/home/polyglot/ziptmp"
if (!file.exists(tmpfolder)) {
  dir.create(tmpfolder)
}

#connect DB and get site name
dbparams <- list(user = "bety", dbname = "bety", password="bety", host="localhost")
host <- list(name = "localhost")
# connect DB and get site name
con      <- db.open(dbparams)

# assign default model according to output file 
model <- unlist(strsplit(outputfile, "\\.met\\."))[-1]
if(identical(model, character(0))){
  if (grepl("\\.ed.zip$", outputfile)) {
    model <- "ED2"
  } else if (grepl("\\.cf$", outputfile)) {
    model <- "LINKAGES"
  } else if (grepl("\\.clim$", outputfile)) {
    model <- "SIPNET"
  } else {
    model <- "BIOCRO"
  }
}

model <- toupper(model)

if (grepl("\\.zip$", inputfile)){
  do.call(file.remove, list(list.files(tmpfolder, full.names = TRUE)))
  unzip (inputfile, exdir = tmpfolder, overwrite = TRUE)
    inputbasenames <- list.files(tmpfolder)
    # get site by file name
    input_met <- list(username = "pecan", source = 'CUSTOM')
    inputnamesplit <- unlist(strsplit(inputbasenames[1], "[_]"))
    for(name in inputnamesplit ) {
      if(grepl("US", name)){
        sitename <- name
      }
    }
    site <- db.query(paste0("SELECT id, sitename AS name FROM sites WHERE sitename LIKE '%", sitename, "%'"),con)
    if(length(site) < 0) {
      # TODO: insert site info
      quit(status=-1)
    } else {
      # remove multiple entries. 
      site <-list(id = site$id[1], name = site$name[1])
    }
    
    years <- c()
    for( file in inputbasenames){ 
      years <- c(years, strtoi(unlist(strsplit(file, "[.]"))[2], base = 0L))
    }
    
    start_year <- min(years)
    end_year <- max(years)
    start_date <- paste0(start_year, "-01-01 00:00:00")
    end_date <- paste0(end_year, "-12-31  23:59:59")
    print("Using met.process to download files")
    outfile_met <-  met.process(site, input_met, start_date, end_date, model, host, dbparams, cacheDir, inputfiles = list.files(tmpfolder, full.names = TRUE))
    
} else{
  #input file is .nc
  inputbasename <- basename(inputfile)
  inputnamesplit <- unlist(strsplit(inputbasename, "[_]"))
  for(name in inputnamesplit ) {
    if(grepl("US", name)){
      sitename <- name
    }
  }
  site <- db.query(paste0("SELECT id, sitename AS name FROM sites WHERE sitename LIKE '%", sitename, "%'"),con)
  if(length(site) < 0) {
    # insert site info
    quit(status=-1)
  } else {
    # remove multiple entries. 
    site <-list(id = site$id[1], name = site$name[1])
  }
  year <- strtoi(unlist(strsplit(inputbasename, "[.]"))[2], base = 0L)
  start_date <- paste0(year, "-01-01 00:00:00")
  end_date <- paste0(year, "-12-31  23:59:59")
  print("Using met.process to download files")
  outfile_met <-  met.process(site, input_met, start_date, end_date, model, host, dbparams, cacheDir, inputfiles = inputfile)
}

db.close(con)
# get start/end year code works on whole years only
start_year <- lubridate::year(start_date)
end_year <- lubridate::year(end_date)

# folder for files with gapfilling 
folder  <- dirname(outfile_met)
outname <- basename(outfile_met)
# get list of files we need to zip by matching years, may need matching outname
files <- c()
for(year in start_year:end_year) {
  files <- c(files, file.path(folder, list.files(folder, pattern = paste0("*", year, "*"))))
}

# use intermediate file so it does not get marked as done until really done
dir.create(tempDir, showWarnings=FALSE, recursive=TRUE)
zipfile <- file.path(tempDir, "temp.zip")
zip(zipfile, files, extras="-j")
# move file should be fast
file.rename(zipfile, outputfile)



